/*==============================================================================
	Title:			DAB converter reference design
	File Name:		main.c
	Date:			2021/3/25
	Copyright:		Headspring Inc.
	Target CPU:		TMS320F28377S (200MHz)
	Target Board:	HSP0179-B00-0002A
	Target Board:	HSP0183-B00-0001
================================================================================*/

/*** Include *******************************************************/
#include <hs_io.h>
#include <hs_math.h>
#include <HSLib_DAB.h>
#include "HSP0179-B00-0002.h"

/*** Define ********************************************************/
/* PWM setting */
#define CARRIERFREQ_HZ		(50000.0F)	/* Switching frequency[Hz] */
#define INTERRUPT_MASK		(2U)		/* Decimation number */
#define DEADTIME_HV_NSEC	(200.0F)	/* Deadtime(HV-side)[nsec] */
#define DEADTIME_LV_NSEC	(600.0F)	/* Deadtime(LV-side)[nsec] */

/* Timer setting */
#define TIMER0INTERVAL_US (1000UL)	/* Timer0 period[usec] */

/* AD setting */
#define VDCHV_GAIN		(468.0F)	/* HV-side voltage AD conversion gain */
#define VDCHV_OFFSET	(468.0F)	/* HV-side voltage AD conversion offset */
#define VDCLV_GAIN		(235.8F)	/* LV-side voltage AD conversion gain */
#define VDCLV_OFFSET	(235.8F)	/* LV-side voltage AD conversion offset */
#define IDCHV_GAIN		(50.0F)		/* HV-side current AD conversion gain */
#define IDCHV_OFFSET	(0.0F)		/* HV-side current AD conversion offset */
#define IDCLV_GAIN		(100.0F)	/* LV-side current AD conversion gain */
#define IDCLV_OFFSET	(0.0F)		/* LV-side current AD conversion offset */

/* Mathematical constant */
#define MATHPI		(3.1415926F)	/*  */
#define MATHSQRT3	(1.732051F)		/* 3 */

/* Controller parameter */
#define VDCLVKP				(0.1F)		/* LV-side voltage Kp */
#define VDCLVTI				(50.0F)		/* LV-side voltage Ti[msec] */
#define IDCLIMIT			(13.0F)		/* LV-side current limit[A] */

/* H/W parameters */
#define TRANS_TURNS_RATIO	(1.867F)		/* Transformer turns ratio */
#define INDUCTANCE_uH		(12.0F)			/* Inductance[uH] */

/*** Typedef *******************************************************/
/* System state */
typedef enum{
	State_Error = 0,				/* 0:Error */
	State_Stop,						/* 1:Stop */
	State_Wait,						/* 2:Wait */
	State_Run,						/* 3:Run */
	State_Precharge,				/* 4:Precharge */
	State_Init,						/* 5:Initialization */
	State_NUM
}SystemState;

/* Protection */
typedef enum{
	Err_IdcLV = 0,					/* 0:IdcLV OC protection*/
	Err_VdcHV,						/* 1:VdcHV OV protection */
	Err_VdcLV,						/* 2:VdcLV OV protection */
	Err_FLTHV,						/* 3:HV-side gate driver error */
	Err_FLTLV,						/* 4:LV-side gate driver error */
	Err_WDT,						/* 5:Watch dog timer error */
	Err_TRIP,						/* 6:Trip state */
	Err_NUM
}ErrState;

/* DI signal */
typedef enum{
	DI_FLTHV = 0,					/* 0:HV-side gate driver error signal */
	DI_FLTLV,						/* 1:LV-side gate driver error signal */
	DI_NUM
}DISignal;


/*** Functions *****************************************************/
void Init(void);					/* Initialization function */
void PWMIntFunc(void);				/* PWM interrupt function */
void Timer0IntFunc(void);			/* Timer0 interrupt function */

void JudgeError(void);				/* Judge error condition */
void SetState(void);				/* State machine */
UINT16 MonitorVdcHVON(void);		/* Monitor HV-side voltage for checking the power supply */
UINT16 MonitorVdcHVOFF(void);		/* Monitor HV-side voltage for checking the power supply */
UINT16 WaitReadyForOperation(void);	/* Wait until ready for operation */
void StartPrecharge(void);			/* Pre-processing before Precharge */
void StartDAB(void);				/* Pre-processing before running */
void StopPrecharge(void);			/* Exit process at stop Precharge */
void StopDAB(void);					/* Exit process at stop running */
void CalibADOffset(void);			/* Calibrate AD offset */
void PWMStartStop(void);			/* PWM start/stop processing */

void CtrlPrecharge(void);			/* LV-side voltage controller for precharge */
void CtrlVdc(void);					/* LV-side voltage controller */
void CalcPhaseShiftAngle(void);		/* Calculate phase-shift angle */

/*** Variables *****************************************************/
UINT32 DIData = 0UL;				/* Digital input state */

/* Sensor value */
FLOAT32	VdcHV = 0.0F;				/* HV-side voltage[V] */
FLOAT32	VdcLV = 0.0F;				/* LV-side voltage[V] */
FLOAT32	IdcHV = 0.0F;				/* HV-side current[A] */
FLOAT32	IdcLV = 0.0F;				/* LV-side current[A] */

/* Calculated value */
FLOAT32 VdcHVRev = 0.0F;			/* Inverse of HV-side DC voltage[1/V] */

/* Sensor value applied LPF */
FLOAT32 VdcHVLPF = 0.0F;
FLOAT32 VdcLVLPF = 0.0F;
FLOAT32 IdcHVLPF = 0.0F;
FLOAT32 IdcLVLPF = 0.0F;

/* LPF cut-off frequency[Hz] */
FLOAT32 VdcHVFc = 500.0F;
FLOAT32 VdcLVFc = 500.0F;
FLOAT32 IdcHVFc = 500.0F;
FLOAT32 IdcLVFc = 500.0F;

/* Coefficient for calculating LPF */
FLOAT32 VdcHVLPFCoeff = 0.0F;
FLOAT32 VdcLVLPFCoeff = 0.0F;
FLOAT32 IdcHVLPFCoeff = 0.0F;
FLOAT32 IdcLVLPFCoeff = 0.0F;

/* Calibration */
FLOAT32 GainAdjVdcHV = 1.0F + 0.005F;	/* Gain correction coefficient (HV-side voltage) */
FLOAT32 GainAdjVdcLV = 1.0F + 0.005F;	/* Gain correction coefficient (LV-side voltage) */
FLOAT32 GainAdjIdcHV = 1.0F;			/* Gain correction coefficient (HV-side current) */
FLOAT32 GainAdjIdcLV = 1.0F;			/* Gain correction coefficient (LV-side current) */
FLOAT32 OffsetAdjVdcHV = 0.0F;			/* Offset correction coefficient[V] (HV-side voltage) */
FLOAT32 OffsetAdjVdcLV = 0.0F;			/* Offset correction coefficient[V] (LV-side voltage) */
FLOAT32 OffsetAdjIdcHV = 0.0F;			/* Offset correction coefficient[A] (HV-side current) */
FLOAT32 OffsetAdjIdcLV = 0.0F;			/* Offset correction coefficient[A] (LV-side current) */
FLOAT32 OffsetAdjIdcHV0 = 0.0F;		/* Temporary variable for calculating OffsetAdjIdcHV */
FLOAT32 OffsetAdjIdcLV0 = 0.0F;		/* Temporary variable for calculating OffsetAdjIdcLV */
UINT16 CntOffset = 0U;				/* Counter used at the offset-compensation */
UINT16 FlagCalibEnd = 0U;			/* 1:Calibration ended, 0:Not calibration ended */

/* Protection */
UINT16 ErrorState = 0U;				/* Error state (non-latch) */
UINT16 ErrorStateLatch = 0U;		/* Error state (latch) */
FLOAT32 ErrThrVdcHVHi_V = 810.0F;	/* HV-side OVP threshold[V] */
FLOAT32 ErrThrVdcLVHi_V = 440.0F;	/* LV-side OVP threshold[V] */
FLOAT32 ErrThrIdcLV_A = 40.0;		/* LV-side OCP threshold[A] */
UINT16 FlagWDT = 0U;				/* 0:WDT overflow not occured, 1:WDT overflow occured */

/* State machine */
SystemState State = State_Init;		/* Current System state */
UINT16 InitEnd = 0U;				/* 0:Initializing, 1:Initialization complete */
FLOAT32 PrechargeThrVdcHV_V = 725.0F;	/* State change threshold[V] (->Precharge) */
FLOAT32 StopThrVdcHV_V = 715.0F;	/* State change threshold[V] (->Stop) */
UINT16 RunStartStopReq = 1U;		/* 0:Run->Wait, 1:Wait->Run */
UINT16 TimerMonitoringVdcHVON = 0U;	/* Counter for measuring time from entry Wait */
UINT16 TimerMonitoringVdcHVOFF = 0U;	/* Counter for measuring time from entry Wait */
UINT16 StepEntryRun = 0U;			/* Step indicating entry Run state */
UINT16 FlagExitRun = 0U;			/* Flag indicating exit Run */
UINT16 StepEntryPrecharge = 0U;		/* Step indicating entry Precharge state */
UINT16 FlagExitPrecharge = 0U;		/* Flag indicating exit Precharge */
UINT16 FlagEntryWait = 0U;			/* Flag indicating Wait state */
UINT16 TimerEntryWait = 0U;			/* Counter for measuring time from entry Wait */

/* DC voltage controller */
FLOAT32 IdcLVLimitHi = IDCLIMIT;	/* IdcLV upperlimit[A] */
FLOAT32 IdcLVLimitLo = -IDCLIMIT;	/* IdcLV lowerlimit[A] */
FLOAT32 VdcLVRef = 0.0F;			/* VdcLV reference value */
FLOAT32 ErrVdcLV = 0.0F;			/* VdcLV error[V] */
FLOAT32 VdcLVKp = VDCLVKP;			/* Proportional gain */
FLOAT32 VdcLVKi = VDCLVKP * INTERRUPT_MASK / (VDCLVTI * 0.001F * CARRIERFREQ_HZ);	/* Integral gain */
FLOAT32 VdcLVTi = VDCLVTI;			/* Time constant [msec] */
FLOAT32 VdcLVPElement  = 0.0F;		/* Proportional term */
FLOAT32 VdcLVIElement  = 0.0F;		/* Integral term */
FLOAT32 VdcLVPIElement = 0.0F;		/* (Proportional term) + (Integral term) */
FLOAT32 IdcLVOut = 0.0F;			/* Control variable */

/* Soft-start (LV-side voltage controller) */
FLOAT32 ConstVdcRef= 400.0F;		/* Target voltage[V] */
FLOAT32 DeltaVdcLVRefRamp = 400.0F / 1.0F * ((FLOAT32)INTERRUPT_MASK / CARRIERFREQ_HZ);	/* Increment value of LV-side voltage reference */

/* PWM duty, phase-shift angle */
FLOAT32 DutyHV = 0.5F;				/* HV-side duty ratio[0.0 ~ 0.5] */
FLOAT32 DutyLV = 0.5F;				/* LV-side duty ratio[0.0 ~ 0.5] */
FLOAT32 DutyPrecharge = 0.5F;		/* HV-side duty ratio[0.0 ~ 0.5] (only for precharge) */
FLOAT32 PhaseShiftAngle = 0.0F;		/* phase-shift angle[-1.0 ~ +1.0]. HV-side to LV-side. */
FLOAT32 PhasePrecharge = 0.0F;		/* phase-shift angle[0.0 ~ +1.0] (only for precharge) */

/* Mathematical constant */
const FLOAT32 MathPi = MATHPI;						/*  */
const FLOAT32 Math2Pi = MATHPI * 2.0F;				/* 2 */
const FLOAT32 MathRev2Pi = 1.0F / (MATHPI * 2.0F);	/* 1/(2) */
const FLOAT32 MathSqrt3 = MATHSQRT3;				/* 3 */
const FLOAT32 MathRevSqrt3 = 1.0F / MATHSQRT3;		/* 1/3 */

/* H/W parameter constant */
const FLOAT32 Inductance_H = INDUCTANCE_uH * 0.001F * 0.001F;


/*=======================================================*/
/**
 * @brief   Main function.
 * @return  None.
 */
void main(void)
{
	Init();		/* Initialize */

	/* Main Loop */
	while(1)
	{
		ClearWDT(0U);				/* Clear WDT counter */
		FlagWDT = ReadFlagWDT();	/* Check WDT overflow */

		DIData = InputPIO();		/* DI read */

		UpdateSCOPE();				/* Communication with HSDT-DP */
	}
}

/*=======================================================*/
/**
 * @brief   Initialization function.
 * @return  None.
 */
void Init(void){

	DisableGlobalInterrupt();						/* Disable Global Interrupt */

	InitializeHSLIB_TypeB();						/* Initialize CPU Setting */
	InitializeBoard();								/* Initialize GPIO Setting */

	/* PWM Setting. */
	InitHrDABPWM(CARRIERFREQ_HZ, DEADTIME_HV_NSEC, DEADTIME_LV_NSEC, 2U, 1U, INTERRUPT_MASK);
	InitInterruptFunctionPWM2(&PWMIntFunc);		/* Initialize Interrupt Function */
	InitEnableInterruptPWM2();

	InitSCOPE();									/* Initialize Scope Function */

	/* Initialize AD Setting */
	InitADCA();
	InitADCB();
	InitADCC();
	InitADCD();
	
	SetupSocADCA(0U,7U,2U,75U);						/* Set up AD0 SOC0(ADCA ADCINA2) Vdc_HV */
	SetupSocADCB(0U,7U,2U,75U);						/* Set up AD2 SOC0(ADCB ADCINB2) Idc_HV */
	SetupSocADCD(0U,7U,14U,75U);					/* Set up AD3 SOC0(ADCD ADCIN14) Vdc_LV */
	SetupSocADCC(0U,7U,2U,75U);						/* Set up AD4 SOC0(ADCC ADCINC2) Idc_LV */
	
	InitRangeOffsetADCA(2U,VDCHV_GAIN,VDCHV_OFFSET);		/* Set range(468.0) and offset(-468.0) for ADCINA2 */
	InitRangeOffsetADCB(2U,IDCHV_GAIN,IDCHV_OFFSET);		/* Set range(50.0) and offset(0.0) for ADCINB2 */
	InitRangeOffsetADCA(14U,VDCLV_GAIN,VDCLV_OFFSET);		/* Set range(235.8) and offset(-235.8) for ADCIN14 */
	InitRangeOffsetADCD(14U,VDCLV_GAIN,VDCLV_OFFSET);		/* Set range(235.8) and offset(-235.8) for ADCIN14 */
	InitRangeOffsetADCC(2U,IDCLV_GAIN,IDCLV_OFFSET);		/* Set range(100.0) and offset(0.0) for ADCINC2 */

	/* Comparator Setting */
	InitCMPSS1(4U, 31U, 21U, 63U);					/* Set up Comparator Subsystem for VdcHV */
	InitCMPSS4(4U, 31U, 21U, 63U);					/* Set up Comparator Subsystem for VdcLV */
	InitCMPSS6(2U, 31U, 21U, 63U);					/* Set up Comparator Subsystem for IdcLV */

	SetupHiOutputCMPSS1(3U);
	SetupHiOutputCMPSS4(3U);
	SetupHiOutputCMPSS6(3U);
	SetupLoOutputCMPSS6(3U);

	SetHivalueCMPSS1(ErrThrVdcHVHi_V, 3.0F);		/* Set threshold for over current protection */
	SetHivalueCMPSS4(ErrThrVdcLVHi_V, 3.0F);		/* Set threshold for over current protection */
	SetHivalueCMPSS6(ErrThrIdcLV_A, 3.0F);			/* Set threshold for over current protection */
	SetLovalueCMPSS6(-ErrThrIdcLV_A, 3.0F);			/* Set threshold for over voltage protection */

	/* Initialize WDT */
	InitWDT(0U, 1U);								/* Overflow time:13.1msec, Stop WDT forbidden */

	/* Timer0 Setting */
	InitTIMER0_IntTime(TIMER0INTERVAL_US);			/* Initialize Timer0 Function */
	InitInterruptFunctionTIMER0(&Timer0IntFunc);	/* Initialize Interrupt Function */
	StartCountTIMER0();								/* Start Timer0 Counter */

	/* Trip-Zone Setting */
	InitEpwmXbarPWM();						/* Initialize ePWM X-BAR */
	SetupInputXbar1(67U);					/* TRIP1, DI0 */
	SetupInputXbar2(68U);					/* TRIP2, DI1 */
	SetupEpwmXbarTRIP4PWM(0xC41UL);			/* TRIP4 : CMPSS1-Hi, CMPSS4-Hi, CMPSS6-Hi/Lo */
	
	InitDCAHTripPWM2(0x000BU);				/* TRIP1,2,4 */
	InitDCAHTripPWM3(0x000BU);				/* TRIP1,2,4 */
	InitDCAHTripPWM5(0x000BU);				/* TRIP1,2,4 */
	InitDCAHTripPWM6(0x000BU);				/* TRIP1,2,4 */
	InitDCTripPWM2(0x000FU, 0x0002U);		/* Trip Combination Input, DCAEVT1.force */
	InitDCTripPWM3(0x000FU, 0x0002U);		/* Trip Combination Input, DCAEVT1.force */
	InitDCTripPWM5(0x000FU, 0x0002U);		/* Trip Combination Input, DCAEVT1.force */
	InitDCTripPWM6(0x000FU, 0x0002U);		/* Trip Combination Input, DCAEVT1.force */

	EnableTripControlPWM2(0x0040U);			/* Set up DCAEVT1 to trip source */
	EnableTripControlPWM3(0x0040U);			/* Set up DCAEVT1 to trip source */
	EnableTripControlPWM5(0x0040U);			/* Set up DCAEVT1 to trip source */
	EnableTripControlPWM6(0x0040U);			/* Set up DCAEVT1 to trip source */

	SetPhaseDutyDABPWM(0.5F, 0.5F, 0.0F);	/* Set Duty ratio and phase-shift angle */
	InitAdsocaPWM2(1U, INTERRUPT_MASK);		/* Initialization of AD-conversion starting trigger */
	EnableAdsocaPWM2();						/* Enable AD trigger */

	wait1us(1000);							/* Wait 1msec */

	/* Clear CMPSSLowLatch/CMPSSHighLatch */
	ClearLatchHiCMPSS1();
	ClearLatchHiCMPSS4();
	ClearLatchHiCMPSS6();
	ClearLatchLoCMPSS6();

	/* Clear trip flag */
	ClearTripFlagPWM2();
	ClearTripFlagPWM3();
	ClearTripFlagPWM5();
	ClearTripFlagPWM6();
	
	EnableInterruptPWM2();					/* Enable PWM2 Interrupt */
	EnableInterruptTIMER0();				/* Enable Timer0 Interrupt */
	EnableGlobalInterrupt();				/* Enable Global Interrupt */

	InitEnd = 1U;							/* Flag for state machine (Init->Stop) */
}

/*=======================================================*/
/**
 * @brief   PWM Interrupt Function.
 * @return  None.
 */
#pragma CODE_SECTION(PWMIntFunc, "ram_code")
#pragma	INTERRUPT(PWMIntFunc)
void PWMIntFunc(void)
{
	AcknowledgePieINT();					/* Clear the PIE interrupt acknowledge flag */

	/* PWM start/stop processing */
	PWMStartStop();

	VdcHV = (GainAdjVdcHV * ReadADCA(0U)) + OffsetAdjVdcHV;		/* Acquire sensor value : VdcHV */
	VdcLV = (GainAdjVdcLV * ReadADCD(0U)) + OffsetAdjVdcLV;		/* Acquire sensor value : VdcLV */
	IdcHV = (GainAdjIdcHV * ReadADCB(0U)) + OffsetAdjIdcHV;		/* Acquire sensor value : IdcHV */
	IdcLV = (GainAdjIdcLV * ReadADCC(0U)) + OffsetAdjIdcLV;		/* Acquire sensor value : IdcLV */

	/* Apply LPF */
	VdcHVLPF = (VdcHVLPFCoeff * VdcHVLPF) + ((1.0F - VdcHVLPFCoeff) * VdcHV);
	VdcLVLPF = (VdcLVLPFCoeff * VdcLVLPF) + ((1.0F - VdcLVLPFCoeff) * VdcLV);
	IdcHVLPF = (IdcHVLPFCoeff * IdcHVLPF) + ((1.0F - IdcHVLPFCoeff) * IdcHV);
	IdcLVLPF = (IdcLVLPFCoeff * IdcLVLPF) + ((1.0F - IdcLVLPFCoeff) * IdcLV);

	if(VdcHVLPF > 1.0F)
	{
		VdcHVRev = 1.0F / VdcHVLPF;
	}
	else
	{
		VdcHVRev = 1.0F;
	}

	if(State == State_Precharge)
	{
		CtrlPrecharge();				/* LV-side DC voltage controller for precharge */
		CalcPhaseShiftAngle();			/* Calculate phase-shift angle */
		SetPrimaryPhaseShiftDABPWM(DutyPrecharge, PhasePrecharge);	/* Set Duty ratio and phase-shift angle */
	}
	else if(State == State_Run)
	{
		CtrlVdc();						/* LV-side DC voltage controller */
		CalcPhaseShiftAngle();			/* Calculate phase-shift angle */
		SetPhaseDutyDABPWM(DutyHV, DutyLV, PhaseShiftAngle);	/* Set Duty ratio and phase-shift angle */
	}
	else
	{
		/* None */
	}

	ClearInterruptFlagPWM2();			/* Clear PWM2 interrupt flag */
}

/*=======================================================*/
/**
 * @brief   Timer0 Interrupt Function.
 * @return  None.
 */
#pragma	INTERRUPT(Timer0IntFunc)
void Timer0IntFunc(void)
{
	
	HSI_IERtable IntEnable;		/* New interrupt enable tables */
	HSI_IERtable Backup;		/* Pre-configuration interrupt enable table */

	AcknowledgePieINT();		/* Clear the PIE interrupt acknowledge flag */

	/* Interrupt enable state setting (for multiple interrupts) */
	IntEnable.pieier[0]  = 0x0000U;
	IntEnable.pieier[1]  = 0x0000U;
	IntEnable.pieier[2]  = 0x0002U;		/* Enable PWM2 interrupt */
	IntEnable.pieier[3]  = 0x0000U;
	IntEnable.pieier[4]  = 0x0000U;
	IntEnable.pieier[5]  = 0x0000U;
	IntEnable.pieier[6]  = 0x0000U;
	IntEnable.pieier[7]  = 0x0000U;
	IntEnable.pieier[8]  = 0x0000U;
	IntEnable.pieier[9]  = 0x0000U;
	IntEnable.pieier[10] = 0x0000U;
	IntEnable.pieier[11] = 0x0000U;
	IntEnable.ier = 0x4000U;			/* Enable Debugging communication interrupt */

	UpdateIerINT(&IntEnable, &Backup);	/* Update interrupt enable state (for multiple interrupts) */
	EnableGlobalInterrupt();			/* Enable Global Interrupt */

	JudgeError();						/* Judge error condition */
	SetState();							/* State machine */

	/* Calculate LPF parameter */
	VdcHVLPFCoeff = 1.0F / (1.0F + ((2.0F * MATHPI * VdcHVFc * INTERRUPT_MASK) / CARRIERFREQ_HZ));
	VdcLVLPFCoeff = 1.0F / (1.0F + ((2.0F * MATHPI * VdcLVFc * INTERRUPT_MASK) / CARRIERFREQ_HZ));
	IdcHVLPFCoeff = 1.0F / (1.0F + ((2.0F * MATHPI * IdcHVFc * INTERRUPT_MASK) / CARRIERFREQ_HZ));
	IdcLVLPFCoeff = 1.0F / (1.0F + ((2.0F * MATHPI * IdcLVFc * INTERRUPT_MASK) / CARRIERFREQ_HZ));

	/* Calculate integral coefficient */
	VdcLVKi = (VdcLVKp * INTERRUPT_MASK) / (VdcLVTi * 0.001F * CARRIERFREQ_HZ);

	if(FlagCalibEnd == 0U)		/* Check calibration ended */
	{
		CalibADOffset();
	}

	DisableGlobalInterrupt();			/* Disable Global Interrupt */
	RestoreIerINT(&Backup);				/* Restore interrupt enabled state */

	ClearInterruptFlagTIMER0();			/* Clear Timer0 interrupt flag */
}

/*=======================================================*/
/**
 * @brief   Judge error condition.
 * @return  None.
 */
void JudgeError(void){
	UINT16 Error = 0U;

	Error |= (GetStatusLolatchCMPSS6() || GetStatusHilatchCMPSS6()) << Err_IdcLV;	/* 0:IdcLV OC protection*/
	Error |= GetStatusHilatchCMPSS1() << Err_VdcHV;									/* 1:VdcHV OV protection */
	Error |= GetStatusHilatchCMPSS4() << Err_VdcLV;									/* 2:VdcLV OV protection */
	Error |= ((DIData >> DI_FLTHV) & 0x0001U) << Err_FLTHV;							/* 3:HV-side gate driver error */
	Error |= ((DIData >> DI_FLTLV) & 0x0001U) << Err_FLTLV;							/* 4:LV-side gate driver error */
	Error |= (FlagWDT & 0x0001U) << Err_WDT;										/* 6:Watch dog timer error */
	Error |= (GetTripFlagPWM2() & 0x0001U) << Err_TRIP;								/* 7:TRIP state */

	ErrorState = Error;
	if((State != State_Init) && (State != State_Error))
	{
		ErrorStateLatch |=  ErrorState;
	}
}

/*=======================================================*/
/**
 * @brief   State machine.
 * @return  None.
 */
void SetState(void){

	switch(State){
	case State_Error:			/* 0:Error */
		break;

	case State_Stop:			/* 1:Stop */
		if(ErrorStateLatch != 0U)		/* Event:Error detection */
		{
			/* Exit Stop */
			/* None */

			State = State_Error;		/* Destination:Error */

			/* Entry Error */
			RunStartStopReq = 0U;
		}
		else if(MonitorVdcHVON() == 1U)		/* Event:DC power supply powered on */
		{
			/* Exit Stop */
			/* None */

			State = State_Precharge;	/* Destination:Precharge */

			/* Entry Precharge */
			StepEntryPrecharge = 1U;
		}
		else
		{
			/* None */
		}
		break;

	case State_Wait:			/* 2:Wait */
		if(ErrorStateLatch != 0U)		/* Event:Error detection */
		{
			/* Exit Wait */
			FlagEntryWait = 0U;
			TimerEntryWait = 0U;

			State = State_Error;		/* Destination:Error */

			/* Entry Error */
			RunStartStopReq = 0U;
		}
		else if(MonitorVdcHVOFF() == 1U)	/* Event:DC power supply powered off */
		{
			/* Exit Wait */
			FlagEntryWait = 0U;
			TimerEntryWait = 0U;

			State = State_Stop;			/* Destination:Stop */

			/* Entry Stop */
			/* None */
		}
		else if(WaitReadyForOperation() == 1U)		/* Event:Run request */
		{
			/* Exit Wait */
			FlagEntryWait = 0U;
			TimerEntryWait = 0U;

			State = State_Run;			/* Destination:Run */

			/* Entry Run */
			StepEntryRun = 1U;
		}
		else
		{
			/* None */
		}
		
		break;

	case State_Run:				/* 3:Run */
		if(ErrorStateLatch != 0)		/* Event:Error detection */
		{
			/* Exit Run */
			FlagExitRun = 1U;
			
			State = State_Error;		/* Destination:Error */
			
			/* Entry Error */
			RunStartStopReq = 0U;
		}
		else if(MonitorVdcHVOFF() == 1U)	/* Event:DC power supply powered off */
		{
			/* Exit Run */
			FlagExitRun = 1U;

			State = State_Stop;			/* Destination:Stop */
			
			/* Entry Stop */
			/* None */
		}
		else if(RunStartStopReq == 0U)		/* Event:Stop Request */
		{
			/* Exit Run */
			FlagExitRun = 1U;

			State = State_Wait;			/* Destination:Wait */
			
			/* Entry Wait */
			FlagEntryWait = 1U;
		}
		else
		{
			/* None */
		}
		
		break;

	case State_Precharge:		/* 4:Precharge */
		if(ErrorStateLatch != 0)		/* Event:Error detection */
		{
			/* Exit Precharge */
			FlagExitPrecharge = 1U;

			State = State_Error;		/* Destination:Error */
			
			/* Entry Error */
			RunStartStopReq = 0U;
		}
		else if(MonitorVdcHVOFF() == 1U)	/* Event:DC power supply powered off */
		{
			/* Exit Precharge */
			FlagExitPrecharge = 1U;

			State = State_Stop;			/* Destination:Stop */
			
			/* Entry Stop */
			/* None */
		}
		else if(VdcLVLPF > (ConstVdcRef * 0.99F))	/* Event:Precharge is finished */
		{
			/* Exit Precharge */
			FlagExitPrecharge = 1U;

			State = State_Wait;			/* Destination:Wait */
			
			/* Entry Wait */
			FlagEntryWait = 1U;
		}
		else
		{
			/* None */
		}
		break;

	case State_Init:				/* 5:Init */
		if(FlagCalibEnd != 0U)			/* Event:Initialization complete */
		{
			/* Exit Init */
			/* None */

			State = State_Stop;			/* Destination:Stop */
			
			/* Entry Stop */
			/* None */
		}
		break;
	default:
		State = State_Error;
		break;
	}
}

/*=======================================================*/
/**
 * @brief   Monitor HV-side voltage for checking the power supply.
 * @return  Condition of the power supply.
 * 			0:powered off
 * 			1:powered on (continue for 1sec)
 */
UINT16 MonitorVdcHVON(void)
{
	UINT16 ret = 0U;

	if(VdcHVLPF > PrechargeThrVdcHV_V)
	{
		TimerMonitoringVdcHVON++;
	}
	else
	{
		TimerMonitoringVdcHVON = 0U;
	}

	if(TimerMonitoringVdcHVON > 1000U)
	{
		ret = 1U;
		TimerMonitoringVdcHVON = 0U;
	}
	
	return ret;
}

/*=======================================================*/
/**
 * @brief   Monitor HV-side voltage for checking the power supply.
 * @return  Condition of the power supply.
 * 			0:powered on
 * 			1:powered off
 */
UINT16 MonitorVdcHVOFF(void)
{
	UINT16 ret = 0U;

	if(VdcHVLPF < StopThrVdcHV_V)
	{
		TimerMonitoringVdcHVOFF++;
	}
	else
	{
		TimerMonitoringVdcHVOFF = 0U;
	}

	if(TimerMonitoringVdcHVOFF > 5U)
	{
		ret = 1U;
		TimerMonitoringVdcHVOFF = 0U;
	}
	
	return ret;
}

/*=======================================================*/
/**
 * @brief   Wait until ready for operation.
 * @return  ready state.
 * 			0:not ready
 * 			1:ready
 */
UINT16 WaitReadyForOperation(void)
{
	UINT16 ret = 0U;

	if(FlagEntryWait == 1U)
	{
		if(TimerEntryWait > 100U)
		{
			FlagEntryWait = 0U;
		}
		else
		{
			TimerEntryWait++;
		}
	}

	if((TimerEntryWait > 100U) && (RunStartStopReq == 1U))
	{
		ret = 1U;
	}
	
	return ret;
}

/*=======================================================*/
/**
 * @brief   Pre-processing before Precharge.
 * @return  None.
 */
#pragma CODE_SECTION(StartPrecharge, "ram_code")
void StartPrecharge(void)
{
	/* Initialization integral term */
	VdcLVIElement = 0.0F;
	VdcLVRef = 0.0F;
	PhasePrecharge = 1.0F;

	SetPrimaryPhaseShiftDABPWM(DutyPrecharge, PhasePrecharge);			/* Set HV-side Duty ratio and phase  */
}

/*=======================================================*/
/**
 * @brief   Pre-processing before running.
 * @return  None.
 */
#pragma CODE_SECTION(StartDAB, "ram_code")
void StartDAB(void)
{
	/* Initialization integral term */
	VdcLVIElement = 0.0F;
	PhaseShiftAngle = 0.0F;

	VdcLVRef = ConstVdcRef;

	SetPhaseDutyDABPWM(DutyHV, DutyLV, PhaseShiftAngle);	/* Set modulation rate */
}

/*=======================================================*/
/**
 * @brief   Exit process at stop Precharge.
 * @return  None.
 */
#pragma CODE_SECTION(StopPrecharge, "ram_code")
void StopPrecharge(void)
{
	StopPWM(0x014UL, 0x014UL);					/* Stop PWM2,3 */
	StopPrimaryPhaseShiftDABPWM();
}

/*=======================================================*/
/**
 * @brief   Exit process at stop running.
 * @return  None.
 */
#pragma CODE_SECTION(StopDAB, "ram_code")
void StopDAB(void)
{
	StopPWM(0x514UL, 0x514UL);					/* Stop PWM2,3,5,6 */
}

/*=======================================================*/
/**
 * @brief   Calibrate AD offset.
 * @return  None.
 */
void CalibADOffset(void)
{
	CntOffset++;
	OffsetAdjIdcHV0 += -(IdcHV * 0.01);
	OffsetAdjIdcLV0 += -(IdcLV * 0.01);

	if(CntOffset > 100U)		/* 100 samples */
	{
		CntOffset = 0U;
		OffsetAdjIdcHV = OffsetAdjIdcHV0;
		OffsetAdjIdcLV = OffsetAdjIdcLV0;
		FlagCalibEnd = 1U;
	}
}

/*=======================================================*/
/**
 * @brief   PWM start/stop processing.
 * @return  None.
 */
void PWMStartStop(void)
{
	/* Precharge */
	if(FlagExitPrecharge == 1U)
	{
		StopPrecharge();
		FlagExitPrecharge = 0U;
	}
	else if(StepEntryPrecharge == 1U)
	{
		StartPrecharge();
		StepEntryPrecharge = 2U;
	}
	else if(StepEntryPrecharge == 2U)
	{
		StartPWM(0x014UL, 0x014UL);								/* Start PWM2,3 */
		StepEntryPrecharge = 0U;
	}
	else
	{
		/* None */
	}

	/* DAB */
	if(FlagExitRun == 1U)
	{
		StopDAB();
		FlagExitRun = 0U;
	}
	else if(StepEntryRun == 1U)
	{
		StartDAB();
		StepEntryRun = 2U;
	}
	else if(StepEntryRun == 2U)
	{
		StartPWM(0x514UL, 0x514UL);								/* Start PWM2,3,5,6 */
		StepEntryRun = 0U;
	}
	else
	{
		/* None */
	}
}

/*=======================================================*/
/**
 * @brief   LV-side voltage controller for precharge.
 * @return  None.
 */
#pragma CODE_SECTION(CtrlPrecharge, "ram_code")
void CtrlPrecharge(void)
{
	/* rate of change of VdcLVRef */
	VdcLVRef += hslimit((ConstVdcRef - VdcLVRef), DeltaVdcLVRefRamp, 0.0F);

	ErrVdcLV = VdcLVRef - VdcLVLPF;			/* Error */
	VdcLVPElement = ErrVdcLV * VdcLVKp;		/* Proportional term */
	VdcLVIElement = hslimit(ErrVdcLV * VdcLVKi + VdcLVIElement, IdcLVLimitHi, 0.0F);		/* Integral term */
	VdcLVPIElement = hslimit(VdcLVPElement + VdcLVIElement, IdcLVLimitHi, 0.0F);
	IdcLVOut = VdcLVPIElement;		/* Controller output (LV-side current) */
	VdcLVIElement = hslimit(IdcLVOut - VdcLVPElement, IdcLVLimitHi, 0.0F);		/* Anti-windup */
}

/*=======================================================*/
/**
 * @brief   LV-side voltage controller.
 * @return  None.
 */
#pragma CODE_SECTION(CtrlVdc, "ram_code")
void CtrlVdc(void)
{
	ErrVdcLV = VdcLVRef - VdcLVLPF;			/* Error */
	VdcLVPElement = ErrVdcLV * VdcLVKp;		/* Proportional term */
	VdcLVIElement = hslimit(ErrVdcLV * VdcLVKi + VdcLVIElement, IdcLVLimitHi, IdcLVLimitLo);		/* Integral term */
	VdcLVPIElement = hslimit(VdcLVPElement + VdcLVIElement, IdcLVLimitHi, IdcLVLimitLo);
	IdcLVOut = VdcLVPIElement;		/* Controller output (LV-side current) */
	VdcLVIElement = hslimit(IdcLVOut - VdcLVPElement, IdcLVLimitHi, IdcLVLimitLo);		/* Anti-windup */
}

/*=======================================================*/
/**
 * @brief   Calculate phase-shift angle.
 * @return  None.
 */
#pragma CODE_SECTION(CalcPhaseShiftAngle, "ram_code")
void CalcPhaseShiftAngle(void)
{
	FLOAT32 Phase_rad = 0.0F;
	FLOAT32 ratio = 1.0F;

	if(State == State_Precharge)
	{
		Phase_rad = IdcLVOut * Math2Pi * Inductance_H * TRANS_TURNS_RATIO * TRANS_TURNS_RATIO * CARRIERFREQ_HZ * VdcHVRev;
		PhasePrecharge = hslimit((1.0F - Phase_rad / MathPi), 0.999F, 0.001F);
	}
	else if(State == State_Run)
	{
		ratio = TRANS_TURNS_RATIO * VdcLVLPF * VdcHVRev;
		if(IdcLVOut > 0.0F)
		{
			Phase_rad = 0.5F * MathPi * (1.0F - hssqrt(1.0F - 8.0F * CARRIERFREQ_HZ * Inductance_H * TRANS_TURNS_RATIO * TRANS_TURNS_RATIO * VdcHVRev / ratio * IdcLVOut));
		}
		else
		{
			Phase_rad = -0.5F * MathPi * (1.0F - hssqrt(1.0F + 8.0F * CARRIERFREQ_HZ * Inductance_H * TRANS_TURNS_RATIO * TRANS_TURNS_RATIO * VdcHVRev / ratio * IdcLVOut));
		}
		
		PhaseShiftAngle = hslimit((Phase_rad / MathPi * 2.0F), 0.999F, -0.999F);
	}
	else
	{
		/* None */
	}
}
